"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
Object.defineProperty(exports, "FederatedTypesPlugin", {
    enumerable: true,
    get: ()=>FederatedTypesPlugin
});
const _interopRequireDefault = require("@swc/helpers/lib/_interop_require_default.js").default;
const _typescript = /*#__PURE__*/ _interopRequireDefault(require("typescript"));
const _download = /*#__PURE__*/ _interopRequireDefault(require("download"));
const _fs = /*#__PURE__*/ _interopRequireDefault(require("fs"));
const _path = /*#__PURE__*/ _interopRequireDefault(require("path"));
const _lodashGet = /*#__PURE__*/ _interopRequireDefault(require("lodash.get"));
const _axios = /*#__PURE__*/ _interopRequireDefault(require("axios"));
let FederatedTypesPlugin = class FederatedTypesPlugin {
    apply(compiler) {
        var ref, ref1;
        let recompileInterval;
        this.webpackCompilerOptions = compiler.options;
        const distPath = (0, _lodashGet.default)(this.webpackCompilerOptions, 'devServer.static.directory') || (0, _lodashGet.default)(this.webpackCompilerOptions, 'output.path') || 'dist';
        this.distDir = _path.default.join(distPath, this.typescriptFolderName);
        this.typesIndexJsonFilePath = _path.default.join(this.distDir, this.typesIndexJsonFileName);
        // The '/' at the end is necessary for TS Compiler to emit files to the output dir.
        this.tsCompilerOptions.outDir = _path.default.join(this.distDir, '/');
        const federationOptions = this.webpackCompilerOptions.plugins.find((plugin)=>{
            return plugin.constructor.name === 'ModuleFederationPlugin';
        });
        const inheritedPluginOptions = (0, _lodashGet.default)(federationOptions, '_options') || null;
        this.exposedComponents = ((ref = this.options) == null ? void 0 : ref.exposes) || inheritedPluginOptions.exposes;
        this.remoteComponents = ((ref1 = this.options) == null ? void 0 : ref1.remotes) || inheritedPluginOptions.remotes;
        compiler.hooks.afterCompile.tap('FederatedTypes', (compilation)=>{
            // Reset and create an Interval to refetch types every 60 seconds
            clearInterval(recompileInterval);
            if (compiler.options.mode === 'development') {
                recompileInterval = setInterval(()=>{
                    this.run(compilation);
                }, 1000 * 60);
            }
            // Runs a compilation immediately
            this.run(compilation);
        });
    }
    run(compilation) {
        if (this.exposedComponents) {
            this.extractTypes();
        }
        // Time to import the remote types
        if (this.remoteComponents) {
            // Get the remote URL origin
            this.importRemoteTypes();
        }
    }
    importRemoteTypes() {
        if (this.remoteComponents) {
            const remoteUrls = Object.entries(this.remoteComponents).map(([remote, entry])=>{
                const [, url] = entry.split('@');
                return {
                    origin: new URL(url != null ? url : entry).origin,
                    remote
                };
            });
            remoteUrls.forEach(({ origin , remote  })=>{
                _axios.default.get(`${origin}/${this.typescriptFolderName}/${this.typesIndexJsonFileName}`).then((indexFileResp)=>{
                    var // Download all the d.ts files mentioned in the index file
                    ref;
                    (ref = indexFileResp.data) == null ? void 0 : ref.forEach((file)=>(0, _download.default)(`${origin}/${this.typescriptFolderName}/${file}`, `${this.typescriptFolderName}/${remote}`, {
                            filename: file
                        }));
                }).catch((e)=>console.log('ERROR fetching/writing types', e));
            });
        }
    }
    extractTypes() {
        if (this.exposedComponents) {
            const normalizedFileNames = Object.values(this.exposedComponents).map((exposed)=>{
                const [rootDir, entry] = exposed.split(/\/(?=[^/]+$)/);
                const cwd = this.webpackCompilerOptions.context || process.cwd();
                const normalizedRootDir = _path.default.resolve(cwd, rootDir);
                const filenameWithExt = this.resolveFilenameWithExtension(normalizedRootDir, entry);
                return _path.default.resolve(cwd, rootDir, filenameWithExt);
            }).filter((entry)=>/\.tsx?$/.test(entry));
            const host = _typescript.default.createCompilerHost(this.tsCompilerOptions);
            const originalWriteFileFn = host.writeFile;
            host.writeFile = (...args)=>{
                const [filename, data] = args;
                this.tsDefinitionFilesObj[filename] = data;
                originalWriteFileFn(...args);
            };
            const program = _typescript.default.createProgram(normalizedFileNames, this.tsCompilerOptions, host);
            const emitResult = program.emit();
            if (!emitResult.emitSkipped) {
                const files = Object.keys(this.tsDefinitionFilesObj).map((file)=>file.slice(`${this.distDir}/`.length));
                _fs.default.writeFile(this.typesIndexJsonFilePath, JSON.stringify(files), (e)=>{
                    if (e) {
                        console.log('Error saving the types index', e);
                    }
                });
            }
        }
    }
    // TODO: this method can be improved for performance
    // For every exposedComponents this method traverse through the directories
    // to automatically resolve the extension.
    // We can cache the already traversed directories which can be used to resolve
    // other 'exposedComponents' falling in the same directory that's already have been traversed.
    resolveFilenameWithExtension(rootDir, entry) {
        // Check path exists and it's a directory
        if (!_fs.default.existsSync(rootDir) || !_fs.default.lstatSync(rootDir).isDirectory()) {
            throw new Error('rootDir must be a directory');
        }
        let filename;
        try {
            // Try to resolve exposed component using index
            const files = _fs.default.readdirSync(_path.default.join(rootDir, entry));
            filename = files.find((file)=>file.split('.')[0] === 'index');
            return `${entry}/${filename}`;
        } catch (err) {
            const files1 = _fs.default.readdirSync(rootDir);
            // Handle case where directory contains similar filenames
            // or where a filename like `Component.base.tsx` is used
            filename = files1.find((file)=>{
                const baseFile = _path.default.basename(file, _path.default.extname(file));
                const baseEntry = _path.default.basename(entry, _path.default.extname(entry));
                return baseFile === baseEntry;
            });
            return filename;
        }
    }
    constructor(options){
        this.tsDefinitionFilesObj = {};
        this.typescriptFolderName = '@mf-typescript';
        this.typesIndexJsonFileName = '__types_index.json';
        this.options = options;
        this.tsCompilerOptions = {
            declaration: true,
            emitDeclarationOnly: true
        };
    }
};

//# sourceMappingURL=Typescript.js.map